#ifndef CLIENT_H
#define CLIENT_H

#include "network/ENet.h"
#include "network/Packet.h"
#include "utils/Error.h"

struct Client final {
    typedef enet_uint16 Port;

private:
    ENetHost* client;
    ENetPeer* connection;

public:
    Client();
    Client(const Client&) = delete;
    Client(Client&&) = delete;
    ~Client();
    Client& operator=(const Client&) = delete;
    Client& operator=(Client&&) = delete;

    Error start();
    Error connect(const char* server, Port port, int timeout);
    void disconnect();

    template<typename T>
    void consumeEvents(T& consumer) {
        ENetEvent e;
        while(enet_host_service(client, &e, 0) > 0) {
            switch(e.type) {
                case ENET_EVENT_TYPE_CONNECT: break;
                case ENET_EVENT_TYPE_DISCONNECT_TIMEOUT:
                case ENET_EVENT_TYPE_DISCONNECT: consumer.onDisconnect(); break;
                case ENET_EVENT_TYPE_NONE: return;
                case ENET_EVENT_TYPE_RECEIVE:
                    InPacket in(e.packet);
                    consumer.onPacket(in);
                    enet_packet_destroy(e.packet);
                    break;
            }
        }
    }

    void send(OutPacket& p);
};

#endif